/** 
 * @namespace   biew
 * @file        reg_form.h
 * @brief       This file contains structure prototypes for embededding new
 *              file formats, disassemblers, translation modes e t.c. in BIEW.
 * @version     -
 * @remark      this source file is part of Binary vIEW project (BIEW).
 *              The Binary vIEW (BIEW) is copyright (C) 1995 Nick Kurshev.
 *              All rights reserved. This software is redistributable under the
 *              licence given in the file "Licence.en" ("Licence.ru" in russian
 *              translation) distributed in the BIEW archive.
 * @note        Requires POSIX compatible development system
 *
 * @author      Nick Kurshev
 * @since       1995
 * @note        Development, fixes and improvements
**/
#ifndef __REG_FORM__H
#define __REG_FORM__H

#ifndef __FILE_INI_RUNTIME_SUPPORT_SYSTEM__
#include "biewlib/file_ini.h"
#endif

#ifdef __cplusplus
#define extern "C" {
#endif

typedef unsigned long __FASTCALL__ (*BinFunc)( void );
typedef tBool         __FASTCALL__ (*ModFunc)( void );

#define APREF_NORMAL    0x0000 /**< Append references in short form if it really present in binary */
#define APREF_USE_TYPE  0x0001 /**< Append references in detail form if it really present in binary */
#define APREF_TRY_LABEL 0x0002 /**< Append references in short form even if it not present in binary (smart method) */

/** 
   Appends dissasembler reference to string.
   * @param str          string buffer for append to
   * @param shift        physical address of field, that required of binding
   * @param flags        see above
   * @param codelen      length of field, that required binding
   * @param r_shift      used only if APPREF_TRY_LABEL mode is set, contains real value of field, that required binding
   * @return             one of RAPREF_* constants (see biewutil.h file for detail)
*/
typedef int __FASTCALL__ (*AppRefs)(char *str,unsigned long shift,int flags,int codelen,unsigned long r_shift);

/***************************************************************\
*  This form registry binary file formats                       *
\***************************************************************/

/** List of DisAssembler Bitness */

#define DAB_USE16   0
#define DAB_USE32   1
#define DAB_USE64   2
#define DAB_USE128  3
#define DAB_USE256  4
#define DAB_AUTO    0xFFFFU /**< never must return! Only for selection */

/** Public symbols classes */

#define SC_LOCAL    0 /**< means: present as entry but not exported */
#define SC_GLOBAL   1 /**< measn: exported entry point */

/** object classes */

#define OC_CODE      0 /**< for code objects */
#define OC_DATA      1 /**< for any data objects */
#define OC_NOOBJECT -1 /**< for non objects (means: relocs, resources, tables ...) */

/**
   Binary plugins state constants: 
   @note
     First called 'check_format' - plugin in UNPLUGGED state.
     Second called 'init' - plugin turn into PLUGGED IN state.
     Last called 'destroy' - plugin turn into UNPLUGGED state.
     These foundamental state is not defined through PS_XXX constants,
     because it meaningless.
     General purpose for PS_XXX state is activating of preemptive memory
     allocation mechanism. Means: when plugin in PS_INACTIVE state (when
     references resolving is 'None') then on low memory it can freed some
     buffers. (This need for prevent of infinity loop of memory
     allocation/freing, that possible in some case).
*/

#define PS_INACTIVE    0
#define PS_ACTIVE      1

typedef struct tag_REGISTRY_BIN
{
  const char * name;        /**< name of binary format */
  const char * prompt[10];  /**< on ALT-Fx selection */
  BinFunc   action[10];  /**< action on ALT-Fx selection */
  tBool   __FASTCALL__ (*check_format)( void ); /**< check for format */
  void    __FASTCALL__ (*init)( void );         /**< init plugin (if check o'k) (constructor) */
  void    __FASTCALL__ (*destroy)( void );      /**< destroy plugin (destructor) */
  BinFunc   showHdr;     /**< if not an MZ style format */
  AppRefs   bind;        /**< for show references */
  void    __FASTCALL__ (*set_state)(int state); /**< called, when activity of
                                      plugin is changed. See above for PS_XXX
                                      constants.
                                      Note: Plugin must support counter of
                                      states. (For multiple call purpose) */

                         /** must return CPU platform, that required by format.
                            Full list of platform please see in
                            plugins/disasm.h file.
                            If this function return -1 then platform is
                            undefined. */
  int     __FASTCALL__ (*query_platform)( void );
                         /** must return DAB_XXX. Quick version for
                            disassembler */
  int     __FASTCALL__ (*query_bitness)(unsigned long);

                         /** for displaying offset within struct in left
                            address column */
  void    __FASTCALL__ (*AddressResolving)(char *,unsigned long);

                         /** converts virtual address to physical (mean file
                            offset). Return 0 if not possible */
  unsigned long __FASTCALL__ (*va2pa)(unsigned long va);

                         /** converts physical address to virtual. seg pointer
                            can be NULL */
  unsigned long __FASTCALL__ (*pa2va)(unsigned long pa);


/*-- Below placed functions for 'put structures' method of save as dialog --*/

                         /** returns public symbol (like: 'GetVersion')
                            to str at file offset - pa.
                            class is filled as print above with SC_XXX constants.
                            return 0 - if no symbol name available
                                       in given direction (as_prev)
                                  !0 - if success return physical address
                                       of public name in given direction */
  unsigned long __FASTCALL__ (*GetPubSym)(char *str,unsigned cb_str,unsigned *class,
                             unsigned long pa,tBool as_prev);

                         /** Determines attributes of object at given physical
                            address (equ file offset).
                            Output:
                            name - filled with object name if possible,
                            start- contains begin of object
                            end  - contains end of object
                            WARNING!!! start of next object always equal of
                            end of current object.
                            class - see above for OC_XXX constants.
                            start and end arguments must fill anyway, and must
                            describe boundaries of current area. (For example:
                            if exe-format - new exe i.e. contains MZ and NEW
                            header and given file offset in old exe stub then
                            start = 0, end = begin of first data or code
                            object).
                            Return: logical number of object or 0 if no object
                            */
  unsigned    __FASTCALL__ (*GetObjAttr)(unsigned long pa,char *name,unsigned cb_name,
                              unsigned long *start,unsigned long *end,int *class,int *bitness);

                         /** Prepares internal buffers for put structures
                            method. Called before GetPubSym and GetObjAttr
                            Arguments:
                            start - start position in the file, requiring for
                            structural disassembler.
                            end - final offset in the file, requiring for
                            structural disassembler.
                            Return:
                            False - success
                            True - an error occured (sample: out of memory)
                            */
  tBool         __FASTCALL__ (*prepare_structs)(unsigned long start,unsigned long end);
                        /** Cleans internal buffers after structural
                           disassembler has stop */
  void          __FASTCALL__ (*drop_structs)( void );
}REGISTRY_BIN;

extern REGISTRY_BIN *detectedFormat;

/***************************************************************\
* This form registry modes of translation file                  *
\***************************************************************/

#define __MF_NONE          0x0000
#define __MF_TEXT          0x0001
#define __MF_USECODEGUIDE  0x0002

typedef struct tag_REGISTRY_MODE
{
  const char *  name;
  const char *  prompt[10];  /**< on Ctrl-Fx selection */
  ModFunc       action[10];  /**< action on Ctrl-Fx selection */
  tBool         __FASTCALL__ (*detect)(void); /**< detects - is file can be
                                                   displayed in given  mode */
  unsigned       flags;          /**< see above */

                                 /** Paint the screen */
  unsigned      __FASTCALL__ (*paint)(unsigned keycode,unsigned textshift);

                                 /** NLS codepage conversion.
                                     Return new size of blocks */
  unsigned      __FASTCALL__ (*convert_cp)(char *,unsigned len);
                                 /** NLS support:
                                     return size of symbol in bytes
                                     for selected codepage */
  unsigned      __FASTCALL__ (*get_symbol_size)( void );
  const char *  __FASTCALL__ (*misckey_name)( void ); /**< F4 key name */
  void          __FASTCALL__ (*misc_action)( void ); /**< F4 action */
  unsigned long __FASTCALL__ (*PrevPageSize)(void);  /**< Get previous page size */
  unsigned long __FASTCALL__ (*CurPageSize)(void);   /**< Get current page size */
  unsigned long __FASTCALL__ (*PrevLineWidth)( void ); /**< Get previous line width */
  unsigned long __FASTCALL__ (*CurLineWidth)( void ); /**< Get current line width */
  void          __FASTCALL__ (*help)( void );         /**< display help about mode */
  void          __FASTCALL__ (*read_ini)( hIniProfile * );  /**< reads biew.ini file if need */
  void          __FASTCALL__ (*save_ini)( hIniProfile * );  /**< writes to biew.ini if need */
  void          __FASTCALL__ (*init)( void );         /**< initialize mode (constructor) */
  void          __FASTCALL__ (*term)( void );         /**< destroy mode (destructor) */
}REGISTRY_MODE;

extern REGISTRY_MODE *activeMode;
extern tBool SelectMode( void );
extern void QuickSelectMode( void );

typedef struct tag_REGISTRY_TOOL
{
  const char *  name;                /**< Tool name */
  void          (*tool)( void );     /**< Tool body */
  void          (*read_ini)( void ); /**< read biew.ini if need */
  void          (*save_ini)( void ); /**< write to biew.ini if need */
}REGISTRY_TOOL;

typedef struct tag_REGISTRY_SYSINFO
{
  const char *  name;                /**< System depended information name */
  void          (*sysinfo)( void );  /**< System depended information body */
  void          (*read_ini)( void ); /**< reads biew.ini if need */
  void          (*save_ini)( void ); /**< writes to biew.ini if need */
}REGISTRY_SYSINFO;

#ifdef __cplusplus
}
#endif

#endif
